def trial_division(n: int) -> list[int]:
    primes = []
    for num in range(2, n + 1):
        is_prime = True
        for divisor in range(2, int(num ** 0.5) + 1):
            if num % divisor == 0:
                is_prime = False
                break
        if is_prime:
            primes.append(num)
    return primes


def sieve_of_eratosthenes(n: int) -> list[int]:
    is_prime = [True for _ in range(n + 1)]
    p = 2
    while p * p <= n:
        if is_prime[p]:
            for i in range(p * p, n + 1, p):
                is_prime[i] = False
        p += 1

    primes = []
    for p in range(2, n + 1):
        if is_prime[p]:
            primes.append(p)
    return primes



def prime_factor_decomp(n: int) -> list[int]:
    factors = []
    divisor = 2

    while n > 1:
        while n % divisor == 0:
            factors.append(divisor)
            n //= divisor
        divisor += 1
    
    return factors

print(prime_factor_decomp(3141592653589))


import random
from modulus_operations import mod_exp

def is_prime_brute_force(n: int) -> bool:
    if n <= 1:
        return False
    elif n <= 3:
        return True
    elif n % 2 == 0 or n % 3 == 0:
        return False

    i = 5
    while i * i <= n:
        if n % i == 0 or n % (i + 2) == 0:
            return False
        i += 6
    return True


def FLT_pt(n: int) -> bool:
    a = random.randint(2, n - 1)
    if mod_exp(a, n, n):
        return False
    else:
        return True


def is_prime_fermat(n: int, iterations = 5):
    if n <= 1:
        return False
    if n <= 3:
        return True

    for _ in range(iterations):
        a = random.randint(2, n - 1)
        if mod_exp(a, n, n) != a:
            return False
    return True


def rabin_miller(n: int, k=5):
    if n <= 1:
        return False
    if n <= 3:
        return True
    if n % 2 == 0:
        return False

    r = 0
    d = n - 1
    while d % 2 == 0:
        d //= 2
        r += 1

    for _ in range(k):
        a = random.randint(2, n - 1)
        x = mod_exp(a, d, n)
        if x == 1 or x == n - 1:
            continue
        for _ in range(r - 1):
            x = (x * x) % n
            if x == n - 1:
                break
        else:
            return False
    return True



from setuptools import setup, find_packages

with open('../README.md', 'r') as readme:
    long_descripton = readme.read()
    
setup(
    name='numbertheory',
    version='0.0.2',
    description='Common number theoretic functions.',
    long_description_content_type='text/markdown',
    long_description=long_descripton,
    packages=find_packages(),
    license='GPL-3.0 License',
    url='https://0xkilty.github.io/number-theory/',
    author='0xKilty',
    author_email='ian.kilty@colostate.edu',
    extras_require={
        'dev': ['twine>=4.0.2']
    },
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Intended Audience :: Developers',
        'Natural Language :: English',
    ],
)


def gcd(a: int, b: int) -> int:
    while b != 0:
        a, b = b, a % b
    return a


def extended_euclidean(a: int, b: int) -> tuple[int, int, int]:
    x_prev, x, y_prev, y = 0, 1, 1, 0
    while b != 0:
        quotient = a // b
        a, b = b, a % b
        x_prev, x = x - quotient * x_prev, x_prev
        y_prev, y = y - quotient * y_prev, y_prev
    return x, y, a


def euler_totient(n: int) -> int:
    if n <= 0:
        return 0
    count = 0
    for i in range(1, n + 1):
        if gcd(n, i) == 1:
            count += 1
    return count


def is_coprime(a: int, b: int) -> bool:
    return gcd(a, b) == 1


def fast_exp(a: int, b: int) -> int:
    res = 1
    while b > 0:
        if b % 2 == 1:
            res *= a
        a *= a
        b //= 2
    return res



from common_functions import extended_euclidean

def mod_exp(a: int, b: int, m: int) -> int:
    res = 1
    a %= m
    while b > 0:
        if b & 1:
            res = (res * a) % m
        b >>= 1
        a = (a * a) % m
    return res


def mod_inverse(a: int, m: int) -> int:
    x, _, gcd = extended_euclidean(a, m)
    if gcd != 1:
        raise ValueError("The modular inverse does not exist.")
    return x % m if x >= 0 else x + m

def chinese_remainder_theorem(moduli: list[int], remainders: list[int]) -> list[int]:
    sum = 0
    prod = 1
    for n_i in moduli:
        prod *= n_i
    for n_i, a_i in zip(moduli, remainders):
        p = prod // n_i
        sum += a_i * mod_inverse(p, n_i) * p
    return sum % prod